/* Name: main.c
 * Author: Bill Dally
 * Copyright: All Rights Reserved - April 2020
 * 
 * Simple Proportional Valve Ventilator
 *
 * Global Variables
 * Input
 *   bpm - beats per minute 8-40
 *   etoix10 - etoi ratio times 10 - 10 to 40
 *   vol - tidal volume in ml
 * 
 * Derived
 *   tcy - cycle time in ms - 60000/bpm
 *   ton - on time in ms - tcy * 1/(1 + etoi)
 *   flow - vol (ml) / ton (ms) 
 *
 * Fixed
 *   trise - rise time in ms
 *   tfall - fall time in ms
 *
 * State
 *   time - time within current cycle in ms
 *
 * Function driven by 1ms RTC 
 *   increment time - wrap at tcy
 *   if time < ton
 *     get current flow - start A/D or read I2C
 *     run control law - driven from A/D interrupt
 *     read pressure and log it
 *
 * UI functions
 *  b - set bpm
 *  e - set etoi x 10
 *  v - set tidal volume
 *  c - run calibration - sweep valve and measure flow
 *  ? - menu
 *     
 * Later - add patient intiated breath
 * Later - ramp flow up and down
 *
 */

#include <avr/io.h>
#include <util/delay.h>
#include "adc.h"
#include "serial.h"
#include "timer.h"
#include "pwm.h"
#include "main.h"
#include "flow.h"
#include "siargo.h"
#include "ui.h"
#include "two_wire.h"

volatile static timer_id_t atimer ;

void init_parameters(void) {
  set_defaults() ; // includes gains
  cflow = 0 ;
  time = 0 ;
  ltime = 0 ;
  log_step = TOCK ;
  run = 0 ;
}

void init_ports(void)
{
  DDRB = 1<<6 ; // PB6 is PWM output - MISO
}

// Check for pressure drop at patient
uint8_t patient_trigger() { // ** placeholder
  return(0) ; 
}

//PID control law with feed forward
int16_t pid_ff(uint16_t flow, uint16_t mflow) {
  o2error = oerror ;
  oerror = error ; // for diff
  error = flow - mflow ; // error signal
  //  d_err = ((error<<1) - oerror - o2error)>>1 ;
  d_err = (error - oerror) + (error - o2error) ;
  i_err = i_err + error ;
  if(i_err > MAX_IERR) i_err = MAX_IERR ;
  if(i_err < MIN_IERR) i_err = MIN_IERR ;
  // maybe do this 32b?
  // int32_t lcflow = flow + (((int32_t)error*pgain + i_err * igain + d_err*dgain) >> GAIN_DIV_LG2) ;
  int16_t cflow = flow + (((error * pgain) + (i_err * igain) + (d_err * dgain)) >> GAIN_DIV_LG2) ;  
  if(cflow < 0) cflow = 0 ;
  if(cflow > MAX_FLOW_MLPS) cflow = MAX_FLOW_MLPS ;
  //put_word(flow); comma() ; put_word(mflow) ; comma() ;
  //put_signed(error) ; comma() ; put_signed(cflow); crlf() ;
  return(cflow) ;
}

// Call every TICK ms
void callback(void) {
  time = time + TICK ;
  if(time >= tcy) {
    uint16_t bvol = (mvol * TOCKdKx64k)>>16 ;
    if(run ==1) { put_word(bvol) ; crlf() ;}
    time = 0 ;
  }
  if((time >= ton) && patient_trigger()) time=0 ;  // ** add patient initiated breathing later
  if(run == 1) {
    mflow = get_flow() ;
    mvol = mvol + mflow ; // ** if no flow sensor, use cflow
    ramp_vol = flow * trise/ (2 * TICK) ; // volume for start of ramp
    if(time == 0) { // first cycle, just feed forward
      tflow = flow_step ;
      i_err = 0 ; // zero integral error
      mvol = 0 ; // zero measured volume each breath
      cflow = pid_ff(tflow, mflow) ;
    } else if(time < trise) {
      i_err = 0 ; // avoid windup during transient
      tflow = tflow + flow_step ;
      cflow = pid_ff(tflow, mflow) ;
      tdown = time ; // for next segment
    } else if(mvol <= (vol_limit - ramp_vol)) {
      tflow = flow ;
      cflow = pid_ff(tflow, mflow) ;
      tdown = time ; // for next segment
    } else { // within ramp of target volume
      i_err = 0 ; // or perhaps reduce gain
      uint16_t tleft = trise - (time - tdown) ; // time left in ramp
      uint32_t rramp_vol = (tflow * tleft)/ (2 * TICK) ; // remaining ramp volume
      if(mvol >= (vol_limit - rramp_vol)) tflow = tflow - flow_step ;
      if (mvol >= vol_limit) tflow = 0 ; // cut to zero if at target volume
      cflow = pid_ff(tflow, mflow) ;
      if(tflow <= 0 ) {
	tflow = 0 ;
	cflow = 0 ;
      } else {
	cflow = pid_ff(tflow, mflow) ;
      }
    }
    set_flow(cflow) ;
    if((log_on == 1) && (log_ptr < MAX_LOG)) {
      if(ltime == 0) {
	flog[log_ptr][0] = tflow ;
	flog[log_ptr][1] = mflow ;
	flog[log_ptr][2] = cflow ;
	flog[log_ptr][3] = error ;
	flog[log_ptr][4] = i_err ;
	flog[log_ptr][5] = d_err ;
	log_ptr++ ; // dot() ;
      }
      ltime++ ;
      if(ltime >= log_step) ltime = 0 ;
    }
  } // if run==0, don't touch pwm for now
}

int main(void)
{
  init_parameters() ;
  init_ports() ;
  adc_init() ;
  serial_init() ;
  timer_init() ;
  pwm_init() ;
  tw_init() ;
  compute_derived() ;
  print_values() ;
  read_f2p_tab() ; // read table from EE PROM
  atimer = timer_start(TICK, callback, PERIODIC); // call sample_callback every TICK ms

  while(1) {
    ui() ;
  }

  return 0;   /* never reached */
}
