// Name: main.c
// RTC call back and variable initialization for open source proportional solenoid valve ventilator
//
// Copyright 2020 Bill Dally
//
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//  

/* 
 * Simple Proportional Valve Ventilator
 *
 * Global Variables
 * Input
 *   bpm - beats per minute 8-40
 *   etoix10 - etoi ratio times 10 - 10 to 40
 *   vol - tidal volume in ml
 *   maxp - maximum pressure
 *   p_trig_on - patient triggering enabled
 *   trigp - patient triggering pressure
 *   min_minute_vol - alarm thresholdm
 * 
 * Derived
 *   tcy - cycle time in ms - 60000/bpm
 *   ton - on time in ms - tcy * 1/(1 + etoi)
 *   flow - vol (ml) / ton (s) - in ml/s
 *
 * Fixed
 *   trise - rise time in ms
 *   tfall - fall time in ms
 *
 * State
 *   time - time within current cycle in ms
 *
 * Function driven by 1ms RTC 
 *   increment time - wrap at tcy
 *   if time < ton
 *     get current flow - start A/D or read I2C
 *     run control law - driven from A/D interrupt
 *     read pressure and log it
 *
 * UI functions
 *  b - set bpm
 *  e - set etoi x 10
 *  v - set tidal volume
 *  c - run calibration - sweep valve and measure flow
 *  ? - menu
 *     
 * Later - add patient intiated breath
 * Later - ramp flow up and down
 *
 */

#include <avr/io.h>
#include <util/delay.h>
#include "adc.h"
#include "serial.h"
#include "timer.h"
#include "pwm.h"
#include "main.h"
#include "flow.h"
#include "siargo.h"
#include "ui.h"
#include "two_wire.h"
#include "nxp_pressure.h"
#include "honeywell_pressure.h"
#include "lcd.h"
#include "control.h"
#include "alarm.h"
#include "watchdog.h"
#include "leds.h"
#include "restore.h"
#include "voltage.h"
#include "solenoid.h"
#include "mux.h"

volatile static timer_id_t atimer ;

void init_parameters(void) {
  set_defaults() ; // includes gains
  cflow = 0 ;
  time = 0 ;
  gtime = 0 ;
  ltime = 0 ;
  log_step = 1 ;
  run = 0 ;
  trun = 0 ;
  front_panel_lock = 0 ;
  b_hist_ptr = 0 ;
  alarm = 0 ; // no alarms set
  ftest = 0 ;
  pf_log_ptr = 0 ;
  pf_log_cnt = 0 ;
  peep = 0 ;
  disconnect_count = 0 ;
}

void init_ports(void)
{

  // Port D - PWM out on 4 and 5, solenoids on 6 and 7
  DDRD = 1<<OC1A_PD | 1<<OC1B_PD | 1<<SOL3_PD | 1<<SOL4_PD ;
  // Port C - LEDs and Alarm
  DDRC = 1<<LED1_PC | 1<<LED2_PC | 1<<LED3_PC | 1<<FLOW_MUX1_PC ;
  // LSB of mux control on port B
  DDRB = 1<<FLOW_MUX0_PB ;
  // Port C - switch inputs
  PORTC = (1<<SW2_C) | (1<<SW3_C) ; // pullups
  // Port B switch input and encoders
  PORTB = (1<<ENC11_B) | (1<<ENC12_B) | (1<<ENC21_B) | (1<<ENC22_B) ; // for pullups

  // for RTC test
  DDRA = 1 ; // A0 is output
}


// Check for pressure drop at patient
uint8_t patient_trigger() {
  if((p_trig_on == 1) && (mpressure < (peep - trigp))) {
    trig_time += TICK ;
    if(trig_time >= TRIG_TIME) return(1) ;
  }
  else {
    trig_time = 0 ;
  }
  return(0) ; 
}

int16_t get_peep() {
  int16_t i ;
  if(p_trig_on == 0) { // passivel patient get current pressure
    return(smpressure) ;
  } else { // reach back
    i = pf_log_ptr - PF_LOG_PEEP_OFFSET ;
    if(i < 0) i += MAX_PF_LOG ;
    return(pf_log[i][0]) ;
  }
}


// Call every TICK ms
void callback(void) {
  // debug pulse to check timing
  PORTA |= 1 ; // pulse PA0 for debug

  // actuall callback starts here
  watchdog_reset() ; // reset watchdog

  // includes internal timer
  update_leds() ; // update LEDs

  // check UI switches every 2 cycles - on odd cycle
  if((time & 0x1) == 1) {
    update_encoders() ; // check encoders and update settings
  }
  
  // reduce load by sampling slow analog channels every 16 cycles
  // do on odd cycle to avoid conflicts on even cycles
  if((time & 0xF) == 3) {
    in_p = honeywell_get_pressure() ; // read input pressure even when not running
    if(BLEND) o2_p = honeywell_get_pressure2() ; // read second hp sensor
    else o2_p = 0 ;
    in_v = get_in_v() ; // read input voltage
  }
  
  time = time + TICK ;
  if(time >= tcy) {
    time = 0 ;
    once = 0 ; // end of breath event hasn't happened once yet
    pip = 0 ; // reset each breath
    mvol = 0 ;
  }
  if(run == 1) {
    //if(time == 0) set_leds(6) ;
    //if(time == ton) set_leds(2) ;
    ma_flow = get_flow(AIR_FLOW_MUX) ; // read air flow
    if(BLEND) {
      mo_flow = get_flow(O2_FLOW_MUX) ; // o2 flow
    } else {
      mo_flow = 0 ;
    }
    mflow = ma_flow + mo_flow ;
    mpressure = nxp_get_pressure() ; // general pressure sensor later

    // check pressure limits
    smpressure = (mpressure + smpressure*PS_CONST) >> PS_SHIFT ; // smooth pressure
    if(smpressure > alarm_pip) {
      alarm_trig(ALARM_MAXP) ;
      trig_pip = smpressure ;
    }
    if(smpressure > ABS_PRESS_LIMIT) { // electronic pressure relief valve
      sol3_off() ;
    }
    if(smpressure < NEG_PRESS_LIMIT) { // electronic one-way valve
      sol3_off() ;
    }
    if(smpressure > pip) pip = smpressure ; // record max inpiritory pressure

    // at time 0 get peep and adjust settings
    if(time == 0) {
      peep = get_peep() ;  // record peep pressure (was oosmpressure)
      if(disconnect_count == 0) {
	if(peep < (set_peep - PEEP_FUZZ)) {
	  set_peep_close += SPC_INC ;
	  if(serial_pf == 0) {
	    serial_put('I') ; crlf() ;
	  }
	}
	else if(peep > (set_peep + PEEP_BIG_ERR)) {
	  set_peep_close = set_peep_close >> 1 ; // divide by 2 on big error
	  if(serial_pf == 0) {
	    serial_put('B') ; crlf() ;
	  }
	}
	else if((peep > (set_peep + PEEP_FUZZ)) && (set_peep_close >= SPC_INC)) {
	  set_peep_close -= SPC_INC ;
	  if(serial_pf == 0) {
	    serial_put('D') ; crlf() ;
	  }
	}
	else if((spc_high) && (set_peep_close >= SPC_INC)) {
	  set_peep_close -= SPC_INC ;
	  if(serial_pf == 0) {
	    serial_put('S') ; crlf() ;
	  }
	}
      }
    }
    mvol = mvol + mflow ; // ** if no flow sensor, use cflow ** if tick not 1, multiply by TICK

    if((p_trig_on == 1) && (time == (tcy-1))) { // check apnea alarm
      apnea_count += 1 ;
      if(apnea_count >= ALARM_APNEA_COUNT) alarm_trig(ALARM_APNEA) ;
    }
    if((time >= tarm) && patient_trigger()) {// patient initiated breathing
      time=tcy ;     // set time to tcy to do time=0 items above next cycle.
      apnea_count = 0 ;
      sol3_on() ;
    }
    
    /*
    if((time < ton) && ((time & 0x3F) == 0)) {
      //uint16_t dmvol = (mvol >> 9) ;
      //put_word(dmvol) ; space() ; put_word(mpressure) ; crlf() ;
      put_word(mflow) ; space() ;
      }*/

    control() ; // sets ca_flow and co_flow
    set_flow(ca_flow, 0) ;
    if(BLEND) {
      set_flow(co_flow, 1) ;
    }

    if((time > ton + TGAP) && (once == 0)) { // after end of breath - run once
      once = 1 ;
      bvol = (mvol * MVOL_VOLx256k)>>18 ;
      bpip = pip ;
      if((run == 1) && (serial_pf == 0)) {
	put_word(bvol) ; space() ;
	put_word(bpip) ; space() ;
	put_word(minute_vol) ; space() ;
	put_word(toff) ; crlf() ;
      }
      check_alarms() ;
    }

    // Sync these variables with text string in dump_log() in ui_common.c
    if((log_on == 1) && (log_ptr < MAX_LOG)) {
      if(ltime == 0) {
	uint16_t mxvol = (mvol * MVOL_VOLx256k)>>18 ;
	flog[log_ptr][0] = ma_flow ;
	flog[log_ptr][1] = mo_flow ;
	flog[log_ptr][2] = mpressure ;
	flog[log_ptr][3] = ca_flow ;
	flog[log_ptr][4] = mxvol ;
	flog[log_ptr][5] = air_pid.err ;
	flog[log_ptr][6] = air_pid.ierr ;
	flog[log_ptr][7] = press_pid.ierr ;
	flog[log_ptr][8] = ps_timer ;
	flog[log_ptr][9] = pstate ;
	flog[log_ptr][10] = ps_flow ;
	log_ptr++ ; // dot() ;
      }
      ltime++ ;
      if(ltime >= log_step) ltime = 0 ;
    }
    if((time & PF_LOG_MASK) == 0) { // 32ms
      if(serial_pf ==  1) {
	put_word(smpressure) ;
	space() ;
	put_word(ma_flow) ;
	space() ;
	put_word(mo_flow) ;
	crlf() ;
      }
      //put_word(ma_flow) ;
      if(pf_log_ptr >= MAX_PF_LOG) pf_log_ptr = 0 ;
      pf_log[pf_log_ptr][0] = smpressure ;
      pf_log[pf_log_ptr][1] = ma_flow ;
      pf_log[pf_log_ptr][2] = mo_flow ;
      pf_log_ptr += 1 ; // next position to write
      pf_log_cnt += 1 ;
      //if(pf_log_cnt >= MAX_PF_LOG) pf_log_cnt = MAX_PF_LOG-1 ;
      if(pf_log_cnt > MAX_PF_LOG) pf_log_cnt = MAX_PF_LOG ;
    }
    if((time & GTICK_MASK) == 0) gtime += 1 ; // increment global time once every GTICK
  } else { // run == 0
    if(ftest == 1) { // flow test using control
      /* ** fix this for two gases 
      mflow = get_flow(AIR_FLOW_MUX) ; // air
      cflow = pid_ff(flow, mflow) ;
      set_flow(cflow, 0) ;
      */
    }
  }

  // delay pressure for peep
  if((time & 0x7F) == 0) { //128ms
    oosmpressure = osmpressure ;
    osmpressure = smpressure ;
  }
    
  // variables to update on 256ms interval - to keep LCD from blurring
  if((time & 0xFF) == 0) {
    xtime = time ;
    xsmpress = smpressure ;
    xmflow = mflow ;
  }
  // could check run 0 here
  if((run == 0) && (trun == 0)) {
    set_pwm0(0) ;
    set_pwm1(0) ;
  }

  // for debug - pulse A low here
  PORTA &= 0xFE ;  // A0 back down
}

int main(void)
{
  uint8_t sr = MCUSR ;
  MCUSR = 0 ; // &= ~(1<<WDRF) ;
  watchdog_disable() ;
  init_parameters() ;
  init_ports() ;
  adc_init() ;
  serial_init() ;
  
  put_string_P(PSTR("SR = ")) ;
  put_byte(sr) ; crlf() ;
  
  set_leds(1) ;
  timer_init() ;
  set_leds(2) ;
  pwm_init() ;
  set_leds(3) ;
  tw_init() ;
  set_leds(4) ;

  
  //siargo_init() ; // if using siargo flow sensor
  nxp_init(); 
  honeywell_pressure_init();
  honeywell_flow_init() ;
  set_leds(5) ;
  lcd_init() ;
  lcd_clear() ;  
  set_leds(6) ;
  compute_derived() ;
  print_values() ;
  set_leds(6) ;
  read_f2p_tab(0) ; // read table from EE PROM
  read_f2p_tab(1) ; // air and O2

  // ** temporarily block restore for testing
  //restore_settings() ;
  put_string_P(PSTR("run = ")) ;
  put_byte(run) ; crlf() ;
  
  set_leds(7) ;
  atimer = timer_start(TICK, callback, PERIODIC); // call sample_callback every TICK ms
  watchdog_init() ; // start watchdog timer - cleared by RTC

  put_string_P(PSTR("PSV Vent:\n\r")) ;
  while(1) {
    ui() ;
  }

  return 0;   /* never reached */
}
