// ui_serial.c
// serial user interface for open-source proportional solenoid valve ventilator
// (c) Bill Dally, April 2020
//
// Copyright 2020 Bill Dally
// 
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <stdio.h>
#include "main.h"
#include "params.h"
#include "serial.h"
#include "ui.h"


void serial_ui(void) {
  uint16_t val ; 
  char c = serial_get() ;
  switch(c) {
    case ' ':
      print_values() ;
      break ;
    case 'b' :
      val = bpm ; // save old value
      bpm = get_val() ;
      if(in_range(bpm, MIN_BPM, MAX_BPM, "bpm")) {
	if(compute_derived()) print_values() ;
        else {
	  flow_exceeded() ;
	  bpm = val ;
	}
      } else bpm = val ;
      break ;
    case 'd' :
      set_defaults() ;
      print_values() ;
      break ;
    case 'e' :
      val = etoix10 ;
      etoix10 = get_val() ;
      if(in_range(etoix10, MIN_ETOI, MAX_ETOI, "e:i x10")) {
	if(compute_derived()) print_values() ;
        else {
	  flow_exceeded() ;
	  etoix10 = val ;
	}
      } else etoix10 = val ;
      break ;
    case 'E': // set PEEP
      val = set_peep ;
      set_peep = get_val() ;
      if(in_range(set_peep, MIN_SPEEP, MAX_SPEEP, "Set Peep")) {
	print_values() ;
      } else set_peep = val ;
      break ;
    case 'f': // FIO2
      if(BLEND == 1) {
	val = fio2 ;
	fio2 = get_val() ;
	if(in_range(fio2, MIN_FIO2, MAX_FIO2, "FIO2")) {
	  print_values() ;
	} else fio2 = val ;
      } else {
	put_string_P(PSTR("No blending on this unit\n\r")) ;
      }
      break ;
    case 'F': // End percent flow - eigths
      val = ps_frac ;
      ps_frac = get_val() ;
      if(in_range(ps_frac, MIN_PSFRAC, MAX_PSFRAC, "Flow Fraction - Eighths")) {
	print_values() ;
      } else ps_frac = val ;
      break ;
    case 'l': // disable logging
      serial_pf = 0 ;
      put_string_P(PSTR("Serial logging disabled\n\r")) ;
      break ;
    case 'L': // enable logging
      serial_pf = 1 ;
      put_string_P(PSTR("Serial logging enabled\n\r")) ;
      break ;
    case 'm': // set mode
      c = serial_get() ;
      switch(c) {
      case 'V':
	mode = MODE_VOL ;
	break ;
      case 'v':
	mode = MODE_VOL_ASSIST ;
	break ;
      case 'P':
	mode = MODE_PRESS ;
	break ;
      case 'p':
	mode = MODE_PRESS_ASSIST ;
	break ;
      default:
	break ;
      }
      break ;
    case 'p':
      val = maxp ; // save old value
      maxp = get_val() ;
      if(in_range(maxp, MIN_MAXP, MAX_MAXP, "maxP")) {
	if(compute_derived()) print_values() ;
        else {
	  flow_exceeded() ;
	  maxp = val ;
	}
      } else maxp = val ;
      break ;
    case 'v':
      val = vol ; // save old value
      vol = get_val() ;
      if(in_range(vol, MIN_VOL, MAX_VOL, "tidal volume")) {
	if(compute_derived()) print_values() ;
        else {
	  flow_exceeded() ;
	  vol = val ;
	}
      } else vol = val ;
      break ;
    case 'r':
      compute_derived() ;
      start_vent() ;
      put_string_P(PSTR("Running Ventilation at:\n\r")) ;
      print_values() ;
      break ;
    case 's':
      stop_vent() ;
      put_string_P(PSTR("Stopping Ventilation\n\r")) ;
      break ;
    case 't':
      val = trigp ;
      trigp = get_val() ;
      if(in_range(trigp, MIN_TRIGP, MAX_TRIGP, "Trigger Pressure")) {
	print_values() ;
      } else trigp = val ;
      break ;
    case 'T':
      c = serial_get() ;
      if(c == 'E') {
	c = serial_get() ;
	if(c == 'S') {
	  test_ui() ;
	}
      }
      if(c == '3') {
	test_v3() ;
      }
      break ;
    case 'U':
      test_v3() ;
      break ;
    case 'a':
      c = serial_get() ;
      switch(c) {
      case 'd': // alarm disconnect pressure //**
	val = pip_disconnect ;
	pip_disconnect = get_val() ;
	if(in_range(pip_disconnect, MIN_PIPD, MAX_PIPD, "Disconnect Pressure")) {
	  print_values() ;
	} else pip_disconnect = val ;
	break ;
      case 'l': // alarm low min vol
	val = low_minute_vol ;
	low_minute_vol = get_val() ;
	if(in_range(low_minute_vol, MIN_LMV, MAX_LMV, "Low Minute Volume")) {
	  print_values() ;
	} else low_minute_vol = val ;
	break ;
      case 'h': // alarm high min vol
	val = high_minute_vol ;
	high_minute_vol = get_val() ;
	if(in_range(high_minute_vol, MIN_HMV, MAX_HMV, "High Minute Volume")) {
	  print_values() ;
	} else high_minute_vol = val ;
	break ;
      case 'p': // alarm pressure
	val = alarm_pip ;
	alarm_pip = get_val() ;
	if(in_range(alarm_pip, MIN_ALARMP, MAX_ALARMP, "Alarm Pressure")) {
	  print_values() ;
	} else alarm_pip = val ;
	break ;
      default:
	break ;
      }
      break ;
    case '?':
    default:
      put_string_P(PSTR("<space> - Print values\n\r")) ;
      put_string_P(PSTR("b - Set breaths per min\n\r")) ;
      put_string_P(PSTR("d - Set default values\n\r")) ; //** - also check print_values
      put_string_P(PSTR("E - Set PEEP in mm H2O\n\r")) ;
      put_string_P(PSTR("e - Set 10x e:i ratio\n\r")) ; 
      put_string_P(PSTR("f - Set FIO2 % 21-100\n\r")) ; 
      put_string_P(PSTR("F - Set % flow to end p breath\n\r")) ;
      put_string_P(PSTR("l - Disable pressure/flow logging\n\r")) ;
      put_string_P(PSTR("L - Enable pressure/flow logging\n\r")) ;
      put_string_P(PSTR("m - Set mode V,v,P,p\n\r")) ; 
      put_string_P(PSTR("o - disable patient trigger\n\r")) ;
      put_string_P(PSTR("p - set max pressure in mm H2O\n\r")) ;
      put_string_P(PSTR("t - set trigger pressure in mm H2O\n\r")) ;
      put_string_P(PSTR("v - Set tidal volume in ml\n\r")) ;
      put_string_P(PSTR("r - Run \n\r")) ;
      put_string_P(PSTR("s - Stop \n\r")) ;
      put_string_P(PSTR("ad - Set alarm disconnect pressure \n\r")) ; //** - all four
      put_string_P(PSTR("ap - Set alarm pressure \n\r")) ;
      put_string_P(PSTR("al - Set alarm low minute volume \n\r")) ;
      put_string_P(PSTR("ah - Set alarm high minute volume \n\r")) ;
      break ;
  }
}
